// Set up Nunchuck
#include <Wire.h>
#include "nunchuck_funcs.h"
byte joy_x,joy_y,zbut,cbut; // Nunchuck Variables
byte data = 0;
#define HALF_DEADBAND 20 //adjust till desired deadband is achieved
#define JOY_Y_HIGH 255
#define JOY_Y_CENTERED 182 // set to whatever "y joystick value is: " on the serial monitor
#define JOY_Y_LOW 46
#define JOY_X_HIGH 255
#define JOY_X_CENTERED 171 // set to whatever "x joystick value is: " on the serial monitor
#define JOY_X_LOW 46
#define ACC 5

// Set up Motors
#include "DualVNH5019MotorShield.h"
DualVNH5019MotorShield md;
#define TOP_SPEED_FORWARD 150
#define TOP_SPEED_BACKWARD 75
#define TOP_TURN_SPEED 75
int throttle, steering;
int rightGoalSpd, leftGoalSpd;
int rightActualSpd = 0;
int leftActualSpd = 0;


// Set up Other
#define DEBUG true // set to false to stop serial messages

void setup()
{
  Serial.begin(9600);
  nunchuck_init(); // initialize nunchu
  md.init(); // initialize motors
}

void loop ()
{
  // Get nunchuck variables
  nunchuck_get_data();
  joy_x  = (byte)nunchuck_joyx(); // ranges from approx 24 - 225
  joy_y  = (byte)nunchuck_joyy(); // ranges from approx 31 - 226
  zbut = (byte)nunchuck_zbutton();
  cbut = (byte)nunchuck_cbutton();
  

    // scale x and y axis so that both are zeroed at natural center of potentiometer, implement deadband
    if(joy_y > JOY_Y_CENTERED + HALF_DEADBAND) //forward
    {
      throttle = map(joy_y, JOY_Y_CENTERED + HALF_DEADBAND, JOY_Y_HIGH, 0, TOP_SPEED_FORWARD);      
      if(joy_x > JOY_X_CENTERED + HALF_DEADBAND) //left
      {
        steering = map(joy_x, JOY_X_CENTERED + HALF_DEADBAND, JOY_X_HIGH, -0, -TOP_TURN_SPEED);// reverse pot coordinates to agree with cartesian        
      }
      else if(joy_x < JOY_X_CENTERED - HALF_DEADBAND) // right
      {
        steering = map(joy_x, JOY_X_CENTERED - HALF_DEADBAND, JOY_X_LOW, 0, TOP_TURN_SPEED);// reverse pot coordinates to agree with cartesian        
      }
      else // in X deadband
      {
        steering = 0;
      }
    }
    else if(joy_y < JOY_Y_CENTERED - HALF_DEADBAND) //reverse
    {
      throttle = map(joy_y, JOY_Y_CENTERED - HALF_DEADBAND, JOY_Y_LOW, -0, -TOP_SPEED_BACKWARD);      
      if(joy_x > JOY_X_CENTERED + HALF_DEADBAND) //left
      {
        steering = map(joy_x, JOY_X_CENTERED + HALF_DEADBAND, JOY_X_HIGH, -0, -TOP_TURN_SPEED);// reverse pot coordinates to agree with cartesian        
      }
      else if(joy_x < JOY_X_CENTERED - HALF_DEADBAND) // right
      {
        steering = map(joy_x, JOY_X_CENTERED - HALF_DEADBAND, JOY_X_LOW, 0, TOP_TURN_SPEED);// reverse pot coordinates to agree with cartesian        
      }
      else // in X deadband
      {
        steering = 0;
      }
    }
    else //in Y deadband
    {
      throttle = 0;
      if(joy_x > JOY_X_CENTERED + HALF_DEADBAND) //left
      {
        steering = map(joy_x, JOY_X_CENTERED + HALF_DEADBAND, JOY_X_HIGH, -0, -TOP_TURN_SPEED);// reverse pot coordinates to agree with cartesian        
      }
      else if(joy_x < JOY_X_CENTERED - HALF_DEADBAND) // right
      {
        steering = map(joy_x, JOY_X_CENTERED - HALF_DEADBAND, JOY_X_LOW, 0, TOP_TURN_SPEED);// reverse pot coordinates to agree with cartesian        
      }
      else // in X deadband
      {
        steering = 0;
      }
    }
    
    if (zbut == 1)  // zbut = "go"
    {  
      //transfer from x,y to l,r 
      rightGoalSpd = throttle + steering;
      leftGoalSpd = throttle - steering;
      
      rightActualSpd = rightAccelerate(rightGoalSpd);
      leftActualSpd = leftAccelerate(leftGoalSpd);
      
      md.setM1Speed(rightActualSpd);
      md.setM2Speed(leftActualSpd);
      stopIfFault();
    }
    
    if (zbut == 1 && cbut == 0) {
      rightActualSpd = 5;
      leftActualSpd = 5;
      md.setM1Speed(rightActualSpd);
      md.setM2Speed(leftActualSpd);
      stopIfFault();
      delay(200);
      rightActualSpd = 0;
      leftActualSpd = 0;
      md.setM1Speed(rightActualSpd);
      md.setM2Speed(leftActualSpd);
    }
    
    if (zbut == 0) {
      rightActualSpd = 0;
      leftActualSpd = 0;
     md.setM1Brake(100);
     md.setM2Brake(100);
    }

  
  //if (DEBUG) {
  // Debug nunchuck variables  
  Serial.print("joy_x: "); Serial.print(joy_x);
  Serial.print("\t joy_y: "); Serial.print(joy_y);
  Serial.print("\t zbut: "); Serial.print(zbut);
  Serial.print("\t cbut: "); Serial.print(cbut);
  
  // Debug Motors
  Serial.print("\t throttle: ");
  Serial.print(throttle);
  Serial.print("\t steering: ");
  Serial.print(steering);
  Serial.print("\t leftGoalSpd: ");
  Serial.print(leftGoalSpd);
  Serial.print("\t leftActualSpd: ");
  Serial.print(leftActualSpd);
  Serial.print("\t rightGoalSpd: ");
  Serial.print(rightGoalSpd);
  Serial.print("\t rightActualSpd: ");
  Serial.println(rightActualSpd);
  
  //delay(250);
  //}
}

int leftAccelerate (int goalSpd) 
{
  if (goalSpd >= 0)
  {
    if (leftActualSpd < goalSpd) 
    { 
      leftActualSpd = leftActualSpd+ACC;
    } 
    if (leftActualSpd > goalSpd)
    {
      //actualSpd--;
      leftActualSpd = leftActualSpd-ACC;
    }
  } 
  else 
  {
    if (leftActualSpd > goalSpd) 
    { 
      leftActualSpd = leftActualSpd-ACC;
    } 
    if (leftActualSpd < goalSpd)
    {
      //actualSpd++;
      leftActualSpd = leftActualSpd+ACC;
    }
  } 
  return leftActualSpd;
}

int rightAccelerate (int goalSpd) 
{
  if (goalSpd >= 0)
  {
    if (rightActualSpd < goalSpd) 
    { 
      rightActualSpd = rightActualSpd+ACC;
    }
    if (rightActualSpd > goalSpd)
    {
      rightActualSpd = rightActualSpd-ACC;
    }
  } 
  else 
  {
    if (rightActualSpd > goalSpd) 
    { 
      rightActualSpd = rightActualSpd-ACC;
    } 
    if (rightActualSpd < goalSpd)
    {
      rightActualSpd = rightActualSpd+ACC;
    }
  } 
  return rightActualSpd;
}

void stopIfFault()
{
  if (md.getM1Fault())
  {
    Serial.println("M1 fault");
    while(1);
  }
  if (md.getM2Fault())
  {
    Serial.println("M2 fault");
    while(1);
  }
}
